﻿unit uActorNodalCombustible;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uCosa, uGlobs, uActores, uFechas, unodoCombustible, uCosaConNombre, uFichasLPD,
  xMatDefs, uauxiliares;

type

  { TActorNodalCombustible }

  TActorNodalCombustible = class(TActor)
    function ReferenciaAlNodoCombustible (nodocomb: TNodoCombustible): boolean; virtual; abstract;
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;

  //TActorUniNodalCombustible es un nodo NO ELÉCTRICO

  TActorUniNodalCombustible = class(TActorNodalCombustible)
  public
    NodoComb: TNodoCombustible;
    Q: TDAOfNReal;   // caudal inyectado al nodo

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; nodocomb: TNodoCombustible);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function ReferenciaAlNodoCombustible (nodocomb: TNodoCombustible): boolean; override;
    procedure PubliVars; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure Free; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;

  end;

  { TActorBiNodalCombustible }

  TActorBiNodalCombustible = class(TActorNodalCombustible)
  public
    NodoCombA, NodoCombB: TNodoCombustible;
    Q_NodoCombA, Q_NodoCombB: TDAOfNReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; NodoCombA, NodoCombB: TNodoCombustible);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function ReferenciaAlNodoCombustible (nodocomb: TNodoCombustible): boolean; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure Free; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;

  end;

procedure AlInicio;
procedure ALFinal;

implementation

{ TActorNodalCombustible }

class function TActorNodalCombustible.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TActorNodalCombustible, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  result:=list;
end;

//--------------------------
// Métodos de TActorUniNodalCombustible
//==========================

constructor TActorUniNodalCombustible.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD;
  nodocomb: TNodoCombustible);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoComb := NodoComb;
end;

constructor TActorUniNodalCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('NodoComb', TCosa(NodoComb), Self);
  f.EjecutarLectura;
end;

procedure TActorUniNodalCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('NodoComb', NodoComb);
end;

function TActorUniNodalCombustible.ReferenciaAlNodoCombustible(
  nodocomb: TNodoCombustible): boolean;
begin
  Result := self.nodocomb = nodocomb;
end;

procedure TActorUniNodalCombustible.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('Q', '[m3/s]', 6, 2, Q, True, True);   //verificar unidades
end;

procedure TActorUniNodalCombustible.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
  setlength(Q, globs.NPostes);
end;


procedure TActorUniNodalCombustible.Free;
begin
  setlength(Q, 0);
  inherited Free;
end;

class function TActorUniNodalCombustible.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TActorUniNodalCombustible, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('NodoComb', TActorUniNodalCombustible(nil).NodoComb, TDataTypeDataRowOfCosa, 1, -1, TNodoCombustible);
  result:=list;
end;

//-------------------------
// Métodos de TActorBiNodalCombustible
//=========================

constructor TActorBiNodalCombustible.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; NodoCombA, NodoCombB: TNodoCombustible);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoCombA := NodoCombA;
  self.NodoCombB := NodoCombB;
end;

constructor TActorBiNodalCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('nodo combustible A', TCosa(NodoCombA), Self);
  f.rdReferencia('nodo combustible B', TCosa(NodoCombB), Self);
  f.EjecutarLectura;
end;

procedure TActorBiNodalCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('nodo combustible A', NodoCombA);
  f.wrReferencia('nodo combustible B', NodoCombB);
end;

function TActorBiNodalCombustible.ReferenciaAlNodoCombustible(
  nodocomb: TNodoCombustible): boolean;
begin
  Result := (nodoCombA = nodocomb) or (nodoCombB = nodocomb);
end;

procedure TActorBiNodalCombustible.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength(Q_NodoCombA, globs.NPostes);
  setlength(Q_NodoCombB, globs.NPostes);
end;

procedure TActorBiNodalCombustible.Free;
begin
  setlength(Q_NodoCombA, 0);
  setlength(Q_NodoCombB, 0);
  inherited Free;
end;

class function TActorBiNodalCombustible.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TActorBiNodalCombustible, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('nodo combustible A', TActorBiNodalCombustible(nil).NodoCombA, TDataTypeDataRowOfCosa, 1, -1, TNodoCombustible);
  list.AddDataColumn('nodo combustible B', TActorBiNodalCombustible(nil).NodoCombB, TDataTypeDataRowOfCosa, 1, -1, TNodoCombustible);
  result:=list;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TActorUniNodalCombustible.ClassName, TActorUniNodalCombustible);
  registrarClaseDeCosa(TActorBiNodalCombustible.ClassName, TActorBiNodalCombustible);

end;

procedure AlFinal;
begin
end;

end.
