unit uGeneradores;

interface

uses
  Classes,
  uNodos,
  uglobs,
  uActorNodal, usimplex,
  ufichasLPD,
  uFuentesAleatorias,
  math,
  xMatDefs, uFechas, uCosaConNombre, uCosa, uversiones;

type
  //Clase Madre de todos los generadores.

  { TGenerador }

  TGenerador = class(TActorUniNodal)
    // si hay cambios en los parmetros dinmicos que hagan cambiar esto
    // debiera tomar cuenta de ello el ActualizarLPD del actor.
    TonCO2xMWh: NReal;
    LowCostMustRun_: boolean;
    CleanDevelopmentMechanism: boolean;


    flg_CalcularGradienteDeInversion: boolean;
    ValorDeEnergiaAlMarginal: NReal;
    GradienteDeInversion: NReal;
    // ( Ingreso_Spot - Ingreso_porEnergia - Ingreso_porDisponibilidad) / Ingreso_porDisponibilidad


    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; nodo: TNodo; TonCO2xMWh: NReal;
      LowCostMustRun, CleanDevelopmentMechanism,
      flg_CalcularGradienteDeInversion: boolean);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    procedure CalcularGradienteDeInversion;

    procedure sim_PrintResultados_UDisp(var fsal: textfile); virtual;

    function NGeneradoresElectricosDisponiblesFlucar: integer; virtual; abstract;

    procedure PubliVars; override;
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa;
      override;

  end;

  { TListaDeGeneradores }

  { TListGenerador }

  TListGenerador = class(TListaDeCosasConNombre)
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa;
      override;
  end;


  { TGeneradorPostizador }

  TGeneradorPostizador = class(TGenerador)
    flg_RestarParaPostizado: boolean;
    PHoraria_PreSorteosPorUnidad: TDAOfNreal;
    PHoraria_PostSorteosTodaLaCentral: TDAOfNreal;
    PPA: TDAOfNReal; // Potencia por Poste Recibida
    EnergiaDisponibleDelPaso: NReal;
    NMaquinasDisponibles: integer;
    fuenteResumirMaxVar: TFuenteAleatoria;
    jRandomVals: TDAOfNInt;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; nodo: TNodo; TonCO2xMWh: NReal;
      LowCostMustRun, CleanDevelopmentMechanism, xflg_RestarParaPostizado,
      xflg_CalcularGradienteDeInversion: boolean);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;

    // en este procedimiento llama a globs.restarParaPostizar.
    {$IFDEF CALC_DEMANDA_NETA}
    procedure prepararPaso_as; override;
    procedure SorteosDelPaso(sortear: boolean); override;
    procedure prepararPaso_ps_pre; override;
    procedure prepararPaso_ps; override;
    {$ENDIF}

    procedure Free; override;
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa;
      override;
  end;

  TDAOfGenerador = array of TGenerador;


  TListaCentralesAguasArriba = class;

  //Todos los generadores hidraulicos heredan de esta clase
  TGeneradorHidraulico = class(TGenerador)
  public
    ivert: integer; //Indice de la variable de vertimiento en el simplex
    QErogado_, QTurbinado, QVertido: NReal;// Caudales erogados, turbinados y
    // vertidos en m3/s
    VErogado, VTurbinado, VVertido: NReal;// Volumenes erogados, turbinados y
    // vertidos en hm3

    Dual_Vertimiento: NReal; // Variable Dual de VVertido

    fEscurrimientoToCaudal: NReal; // factor de conversin de escurrimiento a caudal

    ce: NReal; // coeficiente energtico de las Turbinas(MWh)/(m3/s)
    ce_Bombeo: NReal; // coeficiente energtico de la bomba (MWh)/(m3/s)

    QAportePropio: NReal; // Caudal de aportes propios en m3/s
    VAportePropio: NReal; // [hm3] volumen de aporte en el paso de tiempo

    EnergiaGenerada: NReal; // [MWh] Total de energa generada en el paso
    EnergiaConsumida: NReal;
    // [MWh] Total de energa consumida por el bombeo en el paso

    cv_agua_USD_Hm3_Inc: NReal; // valor del agua entrante en USD/Hm3

    // Retorna la cota del lago para las centrales con embalse y  la cota
    // de toma para las centrales de pasada.


    function CotaAguasArriba: NReal; virtual; abstract;

    // Calcula la cota de la descarga para poder calcular el Salto
    function CotaDescarga: NReal; virtual; abstract;


    // Devuelve la cantidad de variables de Potencia usada por la Central.
    // Si la potencia puede ser diferente en cada poste retorna globs.NPostes
    // Si la potencia se impone igual en todos los postes debe retornar 1.
    function NVariablesPotenica: integer; virtual;

    // es llamada por ResolverEncadenamientos de las centrales aguas abajo
    // para cargar los coeficientes de aportes de la central que reflejan
    // los volmenes erogados en la fila indicada por la central de aguas abajo
    // como su restriccin de balance de volmenes.
    // Este procedimiento, afecta la fila "iresDestino" del simplex agregando
    // los coeficientes para reflejar los aportes y modifica la ltima fila del
    // simplex agregando los coeficientes para valorizar el agua entregada si
    // el parmetro cv_agua_entregada > 0.
    procedure CargarAportesAFilaCentralAguasAbajo(iresDestino: integer;
    // fila en la que cargar los aportes
      coefLlegada: NReal; // coeficiente de llegada por el que multiplicar los aportes
      cv_agua_entregada: NReal;
    // valor del agua a considerar en la central aguas abajo.
      s: TSimplex); virtual;


    // Resuelve los encadenamientos. Se supone que ya fue resuelto el
    // encadenamiento de eventuales centrales Aguas Arriba.
    // Para llamar en orden los ResolverEncadenamientos de las diferentes
    // centrales, la SalaDeJuegos arma una lista ordenada de las centrales.
    procedure ResolverEncadenamientos(s: TSimplex); virtual; abstract;

    // Caclula la reduccin del salto causada por el caudal Erogado
    function ReduccionDeCotaPorCaudalErogado(QErogado: NReal): NReal; virtual; abstract;

    function centralLagoDescarga: TGeneradorHidraulico; virtual; abstract;
    //palfaro@0904171123
    //comento la funcin porque no se estaba usando
    //function centralesAguasArriba : TListaCentralesAguasArriba; virtual; abstract;

    // retorna true si la central tiene vertimiento por poste y false si es porpaso
    function vertimientoPorPoste: boolean; virtual; abstract;
  end;

  { TFichaCentralAguasArriba }

  TFichaCentralAguasArriba = class(TCosa)
    central: TGeneradorHidraulico;
    coef: NReal;
    constructor Create(capa: integer; central: TGeneradorHidraulico; coef: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa;
      override;

  end;

  { TListaCentralesAguasArriba }

  TListaCentralesAguasArriba = class(TListaDeCosas)
  public
    constructor Create(capa: integer);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function add(coeficiente: NReal; central: TGeneradorHidraulico): integer;
      reintroduce;
    procedure replace(coeficienteNuevo: NReal;
      centralNueva, centralVieja: TGeneradorHidraulico);
    function find(clase, nombre: string; var ipos: integer): boolean; overload;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa;
      override;

  end;

procedure AlInicio;
procedure AlFinal;

implementation


{$IFDEF RESUMEN_POSTIZADO}
var
  fdebug: textfile;
{$ENDIF}


{ TListaDeGeneradores }

class function TListGenerador.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
  b: String;
begin
  //list := TDataColumnListOfCosa.Create(TListGenerador, TGenerador, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  //b:=xClaseDeCosa.ClassName;
  list:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  list.ItemListClaseDeCosa:=TGenerador;
  Result := list;
end;



constructor TGenerador.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD; nodo: TNodo;
  TonCO2xMWh: NReal; LowCostMustRun, CleanDevelopmentMechanism,
  flg_CalcularGradienteDeInversion: boolean);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo);
  self.TonCO2xMWh := TonCO2xMWh;
  self.LowCostMustRun_ := LowCostMustRun;
  self.CleanDevelopmentMechanism := CleanDevelopmentMechanism;

  Self.flg_CalcularGradienteDeInversion := flg_CalcularGradienteDeInversion;
  ValorDeEnergiaAlMarginal := 0;
  GradienteDeInversion := 0;

end;



constructor TGenerador.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);

  TonCO2xMWh := 0.0;
  LowCostMustRun_ := True;
  CleanDevelopmentMechanism := False;
  flg_CalcularGradienteDeInversion := False;

  f.IniciarLecturaRetrasada;

  if f.Version >= 62 then
  begin
    f.rd('TonCO2xMWh', TonCO2xMWh);
    f.rd('LowCostMustRun', LowCostMustRun_);
  end;

  if f.Version >= 67 then
    f.rd('CleanDevelopmentMechanism', CleanDevelopmentMechanism);

  if f.Version >= 112 then
    f.rd('flg_CalcularGradienteDeInversion', flg_CalcularGradienteDeInversion);

  f.EjecutarLectura;

end;


procedure TGenerador.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('TonCO2xMWh', TonCO2xMWh);
  f.wr('LowCostMustRun', LowCostMustRun_);
  f.wr('CleanDevelopmentMechanism', CleanDevelopmentMechanism);
  f.wr('flg_CalcularGradienteDeInversion', flg_CalcularGradienteDeInversion);
end;

procedure TGenerador.CalcularGradienteDeInversion;
var
  k: integer;
begin
  ValorDeEnergiaAlMarginal := 0;

  for k := 0 to high(P) do
    ValorDeEnergiaAlMarginal :=
      ValorDeEnergiaAlMarginal + P[k] * globs.DurPos[k] * nodo.cmarg[k];

  if Abs(Ingreso_PorDisponibilidad) > 1E-2 then
    GradienteDeInversion := (ValorDeEnergiaAlMarginal - costoDirectoDelPaso) /
      Ingreso_PorDisponibilidad
  else
    GradienteDeInversion := 0;
end;


procedure TGenerador.PubliVars;
begin
  inherited PubliVars;
  if flg_CalcularGradienteDeInversion then
  begin
    PublicarVariableNR('E*cmg', 'USD', 12, 1, ValorDeEnergiaAlMarginal, True);
    PublicarVariableNR('GradInv', 'p.u.', 12, 4, GradienteDeInversion, True);
  end;
end;



class function TGenerador.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list := TDataColumnListOfCosa.Create(TGenerador, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('TonCO2xMWh', TGenerador(nil).TonCO2xMWh, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('LowCostMustRun', TGenerador(nil).LowCostMustRun_, TDataTypeBooleanOfCosa, 113, -1);
  list.AddDataColumn('CleanDevelopmentMechanism', TGenerador(nil).CleanDevelopmentMechanism, TDataTypeBooleanOfCosa, 113, -1);
  list.AddDataColumn('flg_CalcularGradienteDeInversion', TGenerador(nil).flg_CalcularGradienteDeInversion, TDataTypeBooleanOfCosa, 113, -1);
  Result := list;
end;


constructor TFichaCentralAguasArriba.Create(capa: integer;
  central: TGeneradorHidraulico; coef: NReal);
begin
  inherited Create(capa);
  self.central := central;
  self.coef := coef;
end;


constructor TFichaCentralAguasArriba.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('central', TCosa(central), self);
  f.rd('coef', coef);
  f.EjecutarLectura;
end;


procedure TFichaCentralAguasArriba.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('central', central);
  f.wr('coef', coef);
end;


class function TFichaCentralAguasArriba.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list := TDataColumnListOfCosa.Create(TFichaCentralAguasArriba, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('central', TFichaCentralAguasArriba(nil).central,
    TDataTypeReferenciaOfCosa, 113, -1, TGeneradorHidraulico);
  list.AddDataColumn('coef', TFichaCentralAguasArriba(nil).coef,
    TDataTypeDoubleOfCosa, 113, -1);

  Result := list;
  ;
end;




constructor TListaCentralesAguasArriba.Create(capa: integer);
begin
  inherited Create(capa, 'CentralesAguasArriba');
end;

constructor TListaCentralesAguasArriba.Create_ReadFromText(f: TArchiTexto);
var
  coeficientes: TDAOfNReal;
  n: integer;
  k: integer;
begin
  if f.version < 3 then
  begin
    f.IniciarLecturaRetrasada;
    TCosa.Create_ReadFromText(f);
    f.rd('n', n);
    f.EjecutarLectura;
    f.IniciarLecturaRetrasada;
    lst := TList.Create;
    for k := 0 to n - 1 do
      lst.add(TFichaCentralAguasArriba.Create(capa, nil, 0));
    for k := 0 to n - 1 do
    begin
      f.rdReferencia(':', TCosa(TFichaCentralAguasArriba(lst[k]).central), Self);
      TFichaCentralAguasArriba(lst[k]).central := nil;
    end;
    f.rd('coeficientes', coeficientes);
    f.EjecutarLectura;
    for k := 0 to n - 1 do
      TFichaCentralAguasArriba(lst[k]).coef := coeficientes[k];
  end
  else
  begin
    f.aux_idCarpeta := 'CentralesAguasArriba';
    inherited Create_ReadFromText(f);
  end;
end;


procedure TListaCentralesAguasArriba.WriteToText(f: TArchiTexto);
var
  i: integer;
begin

  // primero hacemos un barrido y eliminamos las fichas con referencia a nil
  // que pudiera habar por ERRORES en el editor. (causa por la que cambiamos a VER3)
  i := 0;
  while (i < lst.Count) do
    if TFichaCentralAguasArriba(lst[i]).central = nil then
      lst.Delete(i)
    else
      Inc(i);

  // ahora si llamamos a TListDeCosas.WriteToText
  inherited WriteToText(f);
end;

procedure TGenerador.sim_PrintResultados_UDisp(var fsal: textfile);
begin
  Write(fsal, #9, paUnidades.nUnidades_Operativas[0]);
end;




constructor TGeneradorPostizador.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD; nodo: TNodo;
  TonCO2xMWh: NReal; LowCostMustRun, CleanDevelopmentMechanism,
  xflg_RestarParaPostizado, xflg_CalcularGradienteDeInversion: boolean);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo,
    TonCO2xMWh, LowCostMustRun, CleanDevelopmentMechanism,
    xflg_CalcularGradienteDeInversion);

  flg_RestarParaPostizado := xflg_RestarParaPostizado;
end;

constructor TGeneradorPostizador.Create_ReadFromText(f: TArchiTexto);
begin

  inherited Create_ReadFromText(f);
  if f.Version >= 110 then
    f.rd('flg_RestarParaPostizado', flg_RestarParaPostizado)
  else
    flg_RestarParaPostizado := False;
end;

procedure TGeneradorPostizador.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('flg_RestarParaPostizado', flg_RestarParaPostizado);
end;

procedure TGeneradorPostizador.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
  setlength(PHoraria_PreSorteosPorUnidad, ceil(globs.HorasDelPaso));
  setlength(PHoraria_PostSorteosTodaLaCentral, ceil(globs.HorasDelPaso));
  setlength(PPA, globs.NPostes);
  fuenteResumirMaxVar := nil;
  setlength(jRandomVals, globs.NPostes);
end;

{$IFDEF CALC_DEMANDA_NETA}
procedure TGeneradorPostizador.prepararPaso_as;
begin
  vclear(PHoraria_PreSorteosPorUnidad);
end;

procedure TGeneradorPostizador.SorteosDelPaso(sortear: boolean);
var
  k: integer;
begin
  if fuenteResumirMaxVar <> nil then
  begin
    for k := 0 to globs.NPostes - 1 do
      jRandomVals[k] := globs.kBasePosete_idxHorasPostizadas[k] +
        fuenteResumirMaxVar.sorteadorUniforme.randomIntRange(0,
        globs.NHorasDelPoste[k] - 1);
  end;
end;


procedure TGeneradorPostizador.prepararPaso_ps_pre;
var
  k: integer;
begin
  for k := 0 to high(PHoraria_PreSorteosPorUnidad) do
    PHoraria_PostSorteosTodaLaCentral[k] := PHoraria_PreSorteosPorUnidad[k] * NMaquinasDisponibles;

  if self.flg_RestarParaPostizado then
    globs.restarPHoraria(PHoraria_PostSorteosTodaLaCentral);
end;


procedure TGeneradorPostizador.prepararPaso_ps;
var
  iPoste, jHora: integer;
  P: NReal;

begin
  if globs.SalaMinutal then
  begin
    PPA[0] := PHoraria_PostSorteosTodaLaCentral[0];
    EnergiaDisponibleDelPaso := PPA[0] * globs.HorasDelPaso;
  end
  else
  begin
    EnergiaDisponibleDelpaso := 0;
    vclear(PPA);
    if fuenteResumirMaxVar = nil then
    begin
      for jHora := 0 to high(globs.idxHorasPostizadas) do
      begin
        iposte := globs.kPosteHorasDelPaso[jHora];
        PPA[iposte] := PPA[iposte] + PHoraria_PostSorteosTodaLaCentral[jHora];
      end;
      for iposte := 0 to globs.NPostes - 1 do
      begin
        EnergiaDisponibleDelPaso := EnergiaDisponibleDelPaso + PPA[iposte];
        PPA[iposte] := PPA[iposte] / globs.Durpos[iposte];
      end;
    end
    else
    begin
      for iposte := 0 to globs.NPostes - 1 do
      begin
        jHora := globs.idxHorasPostizadas[jRandomVals[iposte]];
        P := PHoraria_PostSorteosTodaLaCentral[jHora];
        EnergiaDisponibleDelPaso := EnergiaDisponibleDelpaso + P * globs.DurPos[iposte];
        PPA[Iposte] := P;
      end;
    end;

  {$IFDEF RESUMEN_POSTIZADO}
    Write(fdebug, 'PHoraria_');
    for jHora := 0 to high(PHoraria_) do
      Write(fdebug, #9, PHoraria_[jHora]);
    writeln(fdebug);

    Write(fdebug, 'PHoraria_sort');
    for jHora := 0 to high(PHoraria_) do
      Write(fdebug, #9, PHoraria_[globs.idxHorasPostizadas[jHora]]);
    writeln(fdebug);

    Write(fdebug, 'jRandom');
    for iposte := 0 to globs.NPostes - 1 do
      Write(fdebug, #9, jRandomVals[iposte]);
    writeln(fdebug);

    Write(fdebug, 'PPA');
    for iposte := 0 to globs.NPostes - 1 do
      Write(fdebug, #9, PPA[iposte]);
    writeln(fdebug);
  {$ENDIF}
  end;

end;

{$ENDIF}


procedure TGeneradorPostizador.Free;
begin
  setlength(PHoraria_PreSorteosPorUnidad, 0);
  setlength(PHoraria_PostSorteosTodaLaCentral, 0);
  inherited Free;
end;

class function TGeneradorPostizador.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
begin

  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);

  Result.AddDataColumn('flg_RestarParaPostizado', TGeneradorPostizador(nil).flg_RestarParaPostizado, TDataTypeBooleanOfCosa, 113, -1, TNodo);

end;


function TListaCentralesAguasArriba.find(clase, nombre: string;
  var ipos: integer): boolean;
var
  k: integer;
  buscando: boolean;
  ficha: TFichaCentralAguasArriba;
  cosa: TCosaConNombre;
begin
  buscando := True;
  for k := 0 to lst.Count - 1 do
  begin
    ficha := lst.items[k];
    cosa := ficha.central;
    if (cosa.ClassName = clase) and (cosa.nombre = nombre) then
    begin
      buscando := False;
      ipos := k;
      break;
    end;
  end;
  Result := not buscando;
end;

class function TListaCentralesAguasArriba.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list := TDataColumnListOfCosa.Create(TListaCentralesAguasArriba,TFichaCentralAguasArriba, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result := list;
end;


function TListaCentralesAguasArriba.add(coeficiente: NReal;
  central: TGeneradorHidraulico): integer;
var
  ficha: TFichaCentralAguasArriba;
begin
  ficha := TFichaCentralAguasArriba.Create(central.capa, central, coeficiente);
  Result := lst.Add(ficha);
end;

procedure TListaCentralesAguasArriba.replace(coeficienteNuevo: NReal;
  centralNueva, centralVieja: TGeneradorHidraulico);
var
  i: integer;
  pf: TFichaCentralAguasArriba;
begin
  for i := 0 to lst.Count - 1 do
  begin
    pf := TFichaCentralAguasArriba(lst[i]);
    if pf.central = centralVieja then
    begin
      pf.central := centralNueva;
      pf.coef := coeficienteNuevo;
      break;
    end;
  end;
end;



function TGeneradorHidraulico.NVariablesPotenica: integer;
begin
  Result := globs.NPostes;
end;

procedure TGeneradorHidraulico.CargarAportesAFilaCentralAguasAbajo(
  iresDestino: integer;
  // fila en la que cargar los aportes
  coefLlegada: NReal; // coeficiente de llegada por el que multiplicar los aportes
  cv_agua_entregada: NReal; // valor del agua a considerar en la central aguas abajo.
  s: TSimplex);

var
  menosCofAporte: NReal;
  icof: integer;
  iposte: integer;
  valor: NReal;
  flgValorizarEntrega: boolean;
  NPs: integer;

begin
  flgValorizarEntrega := cv_agua_entregada > 0;
  menosCofAporte := -coefLlegada; // esto es para tener en cuenta que en la
  // restriccin de donde sacamos los volumens los mismos son extracciones y
  // en la que los sumamos son aportes.



  NPs := NVariablesPotenica;


  // consideramos los volmenes turbinados de la central sacndolos
  // de la  restriccin adicional de balance de volmenes que ser la de Vk+1 >=0 en el caso de
  // tratarse de una hidro con embalse o la de -Turbinado -Vertimiento + Aportes = 0
  // para el caso en que se trate de una hidro de pasada
  for iposte := 0 to NPs - 1 do
  begin
    icof := ivar + iposte;
    valor := s.e(ires, icof) * menosCofAporte;
    s.acum_e(iresDestino, icof, valor); // Vk+1 >= 0
    if flgValorizarEntrega then
      s.acum_e(s.nf, icof, valor * cv_agua_entregada);
  end;

  if vertimientoPorPoste then
  begin
    // consideramos el vertimiento
    icof := ivar + NPs;
    for iposte := 0 to globs.NPostes - 1 do
    begin
      valor := s.e(ires, icof + iposte) * menosCofAporte;
      s.acum_e(iresDestino, icof + iposte, valor); // Vk+1 >= 0
      if flgValorizarEntrega then
        s.acum_e(s.nf, icof + iposte, valor * cv_agua_entregada);
    end;
  end
  else
  begin
    // consideramos el vertimiento
    icof := ivar + NPs;
    valor := s.e(ires, icof) * menosCofAporte;
    s.acum_e(iresDestino, icof, valor); // Vk+1 >= 0
    if flgValorizarEntrega then
      s.acum_e(s.nf, icof, valor * cv_agua_entregada);
  end;
end;


procedure AlInicio;
begin
  ucosa.registrarClaseDeCosa(TListaCentralesAguasArriba.ClassName,
    TListaCentralesAguasArriba);

  ucosa.registrarClaseDeCosa(TFichaCentralAguasArriba.ClassName,
    TFichaCentralAguasArriba);

  ucosa.registrarClaseDeCosa(TListGenerador.ClassName, TListGenerador);
end;

procedure AlFinal;
begin
end;

initialization
{$IFDEF RESUMEN_POSTIZADO}
  assignfile(fdebug, 'debug_resumen_postizado.xlt');
  rewrite(fdebug);
{$ENDIF}
finalization
{$IFDEF RESUMEN_POSTIZADO}
  closefile(fdebug);
{$ENDIF}
end.
