unit ubancodebaterias01;

interface

uses
  Classes,
  xMatDefs, uSimplex, ucosa, uGlobs, uFechas,
  ugeneradores,
  uNodos,
  SysUtils,
  uEstados,
  uFichasLPD,
  uconstantesSimSEE,
  uevapUruguay;

resourcestring
  rsBancoDeBaterias01 = 'Banco de Baterías';

type
  TBancoDeBaterias01 = class; // forward declaration



  { TFichaBancoDeBaterias01 }

  TFichaBancoDeBaterias01 = class(TFichaLPD)
  public

    PMax_Gen: NReal;
    PMax_Dem: NReal;
    ren_Gen: NReal;
    ren_Dem: NReal;
    CapacidadMaxima_MWh: NReal;
    fd: NReal;
    tmr: NReal;

    cv_MWh_ValorizadoManual: NReal;


    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      PMax_Gen, PMax_Dem: NReal; ren_Gen, ren_Dem: NReal;
      CapacidadMaxima_MWh: NReal; fd, TMR, cv_MWh_ValorizadoManual: NReal);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure Free; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;




  { TBancoDeBaterias01 }

  TBancoDeBaterias01 = class(TGenerador)
  public
    pa: TFichaBancoDeBaterias01;

    //Variable de Estado
    NDisc: integer; // discretizacion del estad
    Carga_ini: NReal; // valor inicial

    X_Carga: NReal;
    Xs_Carga: NReal;

    cv_MWh: NReal; // valor de la energía almacenada

    // derivada del costo futuro respecto a la carga de la bateria
    dCFdC: NReal;

    Disponible: boolean;


    PMax_Gen, PMax_Dem: NReal;
    PGen, PDem: TDAOfNReal;

    flg_ValorizadoManual: boolean;



    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean; Carga_ini: NReal; NDisc_: integer;
  flg_ValorizadoManual: boolean;
  TonCO2xMWh: NReal; LowCostMustRun, CleanDevelopmentMechanism: boolean);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;



    procedure PrepararMemoria(globs: TGlobs); override;

    procedure RegistrarParametrosDinamicos; override;


    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;

    class function DescClase: string; override;


    //      procedure Sim_Inicio; override;
    procedure Sim_Cronica_Inicio; override;

    function get_pa_FD( kTipoUnidad: integer ): NReal; override;
    function get_pa_TMR( kTipoUnidad: integer ): NReal; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;

    procedure opt_leerSolucion(s: TSimplex); override;
    procedure EvolucionarEstado; override;


    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreRes(ires: integer; var nombre: string): boolean; override;

    procedure PosicionarseEnEstrellita; override;

    procedure ActualizarEstadoGlobal( flg_Xs: boolean ); override;

    procedure optx_nvxs(var ixr, ixd, iauxr, iauxd: integer); override;
    procedure optx_RegistrarVariablesDeEstado(adminEstados: TAdminEstados);
      override;

    procedure Free; override;

    procedure PubliVars; override;

    // imprime tabla de valores del agua en USD/Hm3
    procedure opt_PrintResultados_Encab(var fsal: textfile); override;
    procedure opt_PrintResultados(var fsal: textfile); override;

    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2
      ): TDataColumnListOfCosa; override;
  end;

procedure AlInicio;
procedure AlFinal;

implementation


constructor TBancoDeBaterias01.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean;
  Carga_ini: NReal; NDisc_: integer;
  flg_ValorizadoManual: boolean;
  TonCO2xMWh: NReal;
  LowCostMustRun, CleanDevelopmentMechanism: boolean);

begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo,
    TonCO2xMWh, LowCostMustRun, CleanDevelopmentMechanism,
      flg_CalcularGradienteDeInversion );
  Self.lpd := lpd;

  self.Carga_ini := Carga_ini;
  self.NDisc := NDisc_;
  self.flg_ValorizadoManual:= flg_ValorizadoManual;
end;

constructor TBancoDeBaterias01.Create_ReadFromText(f: TArchiTexto);
begin
  lpd := nil;
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('Carga_ini', Carga_Ini);
  f.rd('NDisc', NDisc);
  if f.Version >= 121 then
    f.rd( 'flg_ValorizadoManual', flg_ValorizadoManual )
  else
    flg_ValorizadoManual:= false;

  f.rd('lpd', TCosa(lpd));
  f.EjecutarLectura;
  pa := nil;
  nodo := nil;
  if lpd <> nil then
    lpd.Propietario := self;
end;

procedure TBancoDeBaterias01.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('Carga_ini', Carga_ini);
  f.wr('NDisc', NDisc);
  f.wr( 'flg_ValorizadoManual', flg_ValorizadoManual );
  f.wr('lpd', TCosa(lpd));
end;

procedure TBancoDeBaterias01.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength( PGen, globs.NPostes );
  setlength( PDem, globs.NPostes );
end;

procedure TBancoDeBaterias01.RegistrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pa, nil);
end;

function TBancoDeBaterias01.InfoAd_: string;
begin
  Result := '';
end;

function TBancoDeBaterias01.PotenciaFirme: NReal;
begin
  Result := 0;
end;

class function TBancoDeBaterias01.DescClase: string;
begin
  Result := rsBancoDeBaterias01;
end;

function TBancoDeBaterias01.get_pa_FD( kTipoUnidad: integer ): NReal;
begin
  result:= pa.fd;
end;

function TBancoDeBaterias01.get_pa_TMR( kTipoUnidad: integer ): NReal;
begin
  result:= pa.tmr;
end;



procedure TBancoDeBaterias01.SorteosDelPaso(sortear: boolean);
begin
  if globs.ObligarDisponibilidad_1_ then
  begin
    Disponible := True;
    PMax_Gen := pa.PMax_Gen;
    PMax_Dem := pa.PMax_Dem;
  end
  else
  begin
    if (sortear) then
    begin
      ActualizarProbabilidadesReparacionYRotura_(pa.fd, pa.tmr);
      Disponible := Sorteos_RepRotUnidades > 0;
      if Disponible then
      begin
        PMax_Gen := pa.PMax_Gen;
        PMax_Dem := pa.PMax_Dem;
      end
      else
      begin
        PMax_Gen := 0;
        PMax_Dem := 0;
      end;
    end
    else
    begin
      Disponible := True;
      PMax_Gen := pa.PMax_Gen * pa.fd;
      PMax_Dem := pa.PMax_Dem * pa.fd;
    end;
  end;
end;

procedure TBancoDeBaterias01.opt_PrintResultados_Encab(var fsal: textfile);
begin
  Write(fsal, #9, FloatToStrF(X_Carga, ffgeneral, 6, 3));
end;

procedure TBancoDeBaterias01.opt_PrintResultados(var fsal: textfile);
begin
  Write(fsal, #9, FloatToStrF(cv_MWh, ffgeneral, 6, 3));
end;

procedure TBancoDeBaterias01.dump_Variables(var f: TextFile; charIndentacion: char);
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f, charIndentacion, 'Carga[MWh]= ', FloatToStrF(X_Carga, ffFixed, 10, 3));
  writeln(f);
end;

class function TBancoDeBaterias01.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TBancoDeBaterias01, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('Carga_ini', TBancoDeBaterias01(nil).Carga_ini, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('NDisc', TBancoDeBaterias01(nil).NDisc, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('flg_ValorizadoManual', TBancoDeBaterias01(nil).flg_ValorizadoManual, TDataTypeBooleanOfCosa, 1, -1);
  list.AddDataColumn('lpd', TBancoDeBaterias01(nil).lpd, TDataTypeDataRowOfCosa, 1, -1, TFichasLPD);

  Result:=list;
end;

procedure TBancoDeBaterias01.PrepararPaso_ps;
var
  res: integer;
  xrpos: NREal;
  cv_Dem, cv_Gen: NReal;
begin

  if flg_ValorizadoManual then
    cv_MWh:= pa.cv_MWh_ValorizadoManual
  else
  begin
    globs.CF.devxr_continuo(
        ixr,globs.kPaso_ + 1, cv_Dem, cv_Gen, res, xrpos );

    case res of
      -1: cv_MWh:= cv_Dem;
      0: cv_MWh:= ( cv_Dem + cv_Gen )/2.0;
      1: cv_MWh:= cv_Gen;
    end;
  end;

end;

procedure TBancoDeBaterias01.opt_nvers(var ivar, ivae, ires: integer);
begin
  self.ivar := ivar;
  self.ires := ires;
  if Disponible then
  begin
    ivar := ivar + 2 * globs.NPostes + 1; // PGen_i ;  PDem_i ; Xs_Carga
    ires := ires + 1; // Restricción dinámica
                      // X_Carga - PGen_i*dupos_i * renGen + PDem_i * dupos_i / renDem  - Xs_Carga = 0
  end;
end;

procedure TBancoDeBaterias01.opt_cargue(s: TSimplex);
var
  ibaseres: integer;
  iposte: integer;
  jres: integer;
  maux: NReal;
begin

  if not Disponible then
    exit;

{$IFDEF SPXCONLOG}
  spx_NombrarVariables(s);
{$ENDIF}

  ibaseres := nodo.ires;

  // Tenemos que aportar a las restricciones de demanda del nodo al que está
  // conectado el generador
  for iposte := 0 to globs.NPostes - 1 do
  begin
    s.pon_e(ibaseres + iposte, ivar + iposte, 1);
    s.pon_e(ibaseres + iposte, ivar + globs.NPostes + iposte, -1);
  end;


 // Xs_Carga = X_Carga -  PGen_i * durpos_i / renGen +  PDem_i * durpos_i * renDem
  for iposte := 0 to globs.NPostes - 1 do
  begin
    s.pon_e(ires, ivar + iposte, - globs.durpos[iposte] / pa.ren_Gen);
    s.pon_e(ires, ivar + globs.NPostes + iposte, globs.durpos[iposte] * pa.ren_Dem);
  end;

  s.pon_e(ires, ivar + 2* globs.NPostes, -1 ); // -Xs_Carga
  s.pon_e(ires, s.nc, X_Carga);


 // Aportes a la función de -costo.
 for iposte := 0 to globs.NPostes - 1 do
 begin
   s.pon_e( s.nf, ivar + iposte,  s.e( ires, ivar+iposte) * cv_MWh );
   s.pon_e( s.nf, ivar + globs.NPostes + iposte, s.e( ires, ivar+ globs.NPostes +iposte) * cv_MWh );
 end;

end;

procedure TBancoDeBaterias01.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  // Le fijamos como cota máxima PMax a la potencia en todos los postes
  if Disponible then
  begin
    for iposte := 0 to globs.NPostes - 1 do
      s.cota_sup_set(ivar + iposte, PMax_Gen);

    for iposte := 0 to globs.NPostes - 1 do
      s.cota_sup_set(ivar + globs.NPostes + iposte, PMax_Dem);

    s.cota_sup_set( ivar + 2* globs.NPostes, pa.CapacidadMaxima_MWh );
    s.FijarRestriccionIgualdad( ires );
  end;
end;

procedure TBancoDeBaterias01.Sim_Cronica_Inicio;
begin
  inherited Sim_Cronica_Inicio;

  X_Carga := Carga_ini;

end;

procedure TBancoDeBaterias01.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;

begin
  CostoDirectoDelPaso := 0;

  if not disponible then
  begin
    vclear(PGen);
    vclear(PDem);
    vclear(P);
    Xs_Carga := X_Carga;
    exit;
  end;

  for iposte := 0 to globs.NPostes - 1 do
  begin
    PGen[iposte]:= s.xval(ivar + iposte);
    PDem[iposte]:= s.xval(ivar + globs.npostes + iposte);
    P[iposte]:= PGen[iposte]- PDem[iposte];
  end;

  Xs_Carga := s.xval( ivar + 2 * globs.NPostes );

end;


procedure TBancoDeBaterias01.EvolucionarEstado;
begin
  X_Carga := Xs_Carga;
end;


function TBancoDeBaterias01.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar +2* globs.NPostes) then
  begin
    Result := True;
    if (ivar < self.ivar + globs.NPostes) then
      nombre := self.Nombre + '_PGen[MW]' + IntToStr(ivar - self.ivar + 1)
    else
      nombre := self.Nombre + '_PDem[MW]' + IntToStr(ivar - self.ivar - globs.NPostes + 1);
  end
  else
    Result := False;
end;

function TBancoDeBaterias01.getNombreRes(ires: integer; var nombre: string): boolean;
var
  k: integer;
begin
  if (ires = self.ires) then
  begin
    nombre := self.nombre + 'f(x,u,r)-Xs=0';
    result:= true;
  end
  else
    Result := False;
end;

procedure TBancoDeBaterias01.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('PGen', '[MW]', 6, 2, PGen, True, True);
  PublicarVariableVR('PDem', '[MW]', 6, 2, PDem, True, True);
  PublicarVariableNR('cv_MWh', '[USD/MWh]', 6, 1, cv_MWh, True);
  PublicarVariableNR('X_Carga', '[MWh]', 6, 1, X_Carga, True);
  PublicarVariableB('Disponible', '-', Disponible, False);
end;

procedure TBancoDeBaterias01.optx_nvxs(var ixr, ixd, iauxr, iauxd: integer);
begin
  if not flg_ValorizadoManual then
  begin
    self.ixr := ixr;
    ixr := ixr + 1;
  end;
end;

procedure TBancoDeBaterias01.PosicionarseEnEstrellita;
begin
  if not flg_ValorizadoManual then
    X_Carga := globs.CF.xr[ixr];
end;

procedure TBancoDeBaterias01.ActualizarEstadoGlobal( flg_Xs: boolean );
begin
  if not flg_ValorizadoManual then
    globs.CF.xr[ixr] := X_Carga;
end;

procedure TBancoDeBaterias01.optx_RegistrarVariablesDeEstado(
  adminEstados: TAdminEstados);
begin
 if not flg_ValorizadoManual then
  adminEstados.Registrar_Continua(
    ixr,
    0.0,
    pa.CapacidadMaxima_MWh,
    ndisc,
    nombre + '_Carga', // nombre de la variable
    'MWh' // unidades
    );
end;

procedure TBancoDeBaterias01.Free;
begin
  if lpd <> nil then
    lpd.Free;
  inherited Free;
  setlength( PGen, 0 );
  setlength( PDem, 0 );
end;



{*********************************
*Métodos de TFichaBancoBaterias *
*********************************}

constructor TFichaBancoDeBaterias01.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; PMax_Gen, PMax_Dem: NReal; ren_Gen, ren_Dem: NReal;
  CapacidadMaxima_MWh: NReal; fd, TMR, cv_MWh_ValorizadoManual: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  Self.PMax_Gen := PMax_Gen;
  self.PMax_Dem := PMax_Dem;
  self.ren_Gen := ren_Gen;
  self.ren_Dem := ren_Dem;
  self.CapacidadMaxima_MWh := CapacidadMaxima_MWh;
  self.fd := fd;
  self.TMR := TMR;
  self.cv_MWh_ValorizadoManual:= cv_MWh_ValorizadoManual;
end;

procedure TFichaBancoDeBaterias01.WriteToText(f: TArchiTexto);
begin

  inherited WriteToText(f);

  f.wr('PMax_Gen', PMax_Gen);
  f.wr('PMax_Dem', PMax_Dem);
  f.wr('ren_Gen', ren_Gen);
  f.wr('ren_Dem', ren_Dem);
  f.wr('CapacidadMaxima_MWh', CapacidadMaxima_MWh);
  f.wr('fd', fd);
  f.wr('TMR', TMR);
  f.wr( 'cv_MWh_ValorizadoManual', cv_MWh_ValorizadoManual );
end;

constructor TFichaBancoDeBaterias01.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);

  f.IniciarLecturaRetrasada;
  f.rd('PMax_Gen', PMax_Gen);
  f.rd('PMax_Dem', PMax_Dem);
  f.rd('ren_Gen', ren_Gen);
  f.rd('ren_Dem', ren_Dem);
  f.rd('CapacidadMaxima_MWh', CapacidadMaxima_MWh);
  f.rd('fd', fd);
  f.rd('TMR', TMR);

  if f.Version >= 121 then
    f.rd( 'cv_MWh_ValorizadoManual', cv_MWh_ValorizadoManual )
  else
    cv_MWh_ValorizadoManual:= 0.0;

  f.EjecutarLectura;
end;

procedure TFichaBancoDeBaterias01.Free;
begin
  inherited Free;
end;

class function TFichaBancoDeBaterias01.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TFichaBancoDeBaterias01, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('ren_Gen', TFichaBancoDeBaterias01(nil).ren_Gen, TDataTypeDoubleOfCosa, ['', '-'], 1, -1);
  list.AddDataColumn('ren_Dem', TFichaBancoDeBaterias01(nil).ren_Dem, TDataTypeDoubleOfCosa, ['', '-'], 1, -1);
  list.AddDataColumn('CapacidadMaxima_MWh', TFichaBancoDeBaterias01(nil).CapacidadMaxima_MWh, TDataTypeDoubleOfCosa, ['', '-'], 1, -1);
  list.AddDataColumn('fd', TFichaBancoDeBaterias01(nil).fd, TDataTypeDoubleOfCosa, ['', '-'], 1, -1);
  list.AddDataColumn('TMR', TFichaBancoDeBaterias01(nil).TMR, TDataTypeDoubleOfCosa, ['', '-'], 1, -1);
  list.AddDataColumn('cv_MWh_ValorizadoManual', TFichaBancoDeBaterias01(nil).cv_MWh_ValorizadoManual, TDataTypeDoubleOfCosa, ['', '-'], 1, -1);

  Result:=list;

end;


procedure AlInicio;
begin
  registrarClaseDeCosa(TBancoDeBaterias01.ClassName, TBancoDeBaterias01);
  registrarClaseDeCosa(TFichaBancoDeBaterias01.ClassName, TFichaBancoDeBaterias01);
end;

procedure AlFinal;
begin
end;

end.
