unit udemandas01;
{$MODE Delphi}

interface

uses
  SysUtils,
  uauxiliares,
  xmatdefs,
  udemandas, uFuentesAleatorias,
  uNodos, usimplex, ufichasLPD,
  ufechas,
  ucosa,
  uconstantesSimSEE, math01;

resourcestring
  rs3CurvasHorararias = '3 Curvas Horarias';

type

  { TFichaDemanda01 }

  TFichaDemanda01 = class(TFichaLPD)
  public
    PotDiaHabil, PotDiaMedioFeriado, PotDiaFeriado: TDAOfNreal; // Potencia en MW
    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      PotDiaHabil, PotDiaMedioFeriado, PotDiaFeriado: TDAOfNreal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function infoAd_: string; override;
    procedure Free; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2
      ): TDataColumnListOfCosa; override;
  end;


type

  { TDemanda01 }

  TDemanda01 = class(TDemanda)
  public
    pA, pB: TFichaDemanda01; // Fichas de parámetros dinámicos

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; nodo: TNodo; falla_profundidad,
  falla_costo: TDAOfNReal; fuente: TFuenteAleatoria; nombreBorne: string;
  SumarEnergiaHr, flg_SumarParaPostizado: boolean;
  icf_Fuente: TFuenteAleatoria; icf_NombreBorne: string);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure RegistrarParametrosDinamicos; override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    function PotenciaFirme: NReal; override;

    //      procedure prepararSim; override;
    procedure prepararPaso_as; override;
    procedure Free; override;
  private
    CacheActivo: boolean;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2
      ): TDataColumnListOfCosa; override;
  end;


(* Deforma una curva de potencia horaria aplicando factores de crecimiento de
energía (fce) y del pico (fcp).
Por ejemplo, si el pico decrese 10% y la energía debe aumentar 5% los factores
serán fce= 1.05 y fcp= 0.9

El resultado es TRUE si todas las potencias resultantes son positivas y false
si alguna quedó negativa.
*)
function aplicarCrecimientos_CurvaHoraria(var PotHoraria: TDAOfNReal;
  fce, fcp: NReal): boolean;

procedure AlInicio;
procedure AlFinal;

implementation


//----------------------
// Métodos de TDemanda01
//======================

constructor TDemanda01.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades, lpd: TFichasLPD; nodo: TNodo; falla_profundidad, falla_costo: TDAOfNReal;
  fuente: TFuenteAleatoria; nombreBorne: string; SumarEnergiaHr, flg_SumarParaPostizado: boolean;
  icf_Fuente: TFuenteAleatoria; icf_NombreBorne: string);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo,
    falla_profundidad, falla_costo, fuente, nombreBorne,
    icf_fuente, icf_NombreBorne, SumarEnergiaHr, flg_SumarParaPostizado );

  self.lpd := lpd;
  self.fuente := fuente;
  self.nombreBorne := nombreBorne;
  CacheActivo := False;
end;

constructor TDemanda01.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.rd('lpd', TCosa(lpd));
  f.EjecutarLectura;
  pa := nil;
  CacheActivo := False;
  lpd.Propietario := self;
end;

procedure TDemanda01.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('lpd', lpd);
end;


procedure TDemanda01.RegistrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, @pB);
end;

function TDemanda01.InfoAd_: string;
begin
  Result := '';
end;

class function TDemanda01.DescClase: string;
begin
  Result := rs3CurvasHorararias;
end;

function TDemanda01.PotenciaFirme: NReal;
begin
  Result := paUnidades.nUnidades_Operativas[0] * vmax(self.PPa);
end;

procedure TDemanda01.prepararPaso_as;
var
  alfa: NReal;

  hdd: integer;
  tipoDeDia: integer;
  iHoraDelPaso: integer;
  instante: TFecha;
begin
{  if CacheActivo then exit;
  CacheActivo:= true;}


  instante:= globs.FechaInicioDelpaso;

  alfa := (instante.dt - pA.fecha.dt) / ( pB.fecha.dt - pA.fecha.dt );
  hdd := instante.hora;

  for iHoraDelPaso := 0 to high(PHoraria) do
  begin
    tipoDeDia := instante.TipoDeDia;
    case tipoDeDia of
      0: PHoraria[iHoraDelPaso] :=
          ponder(pA.PotDiaHabil[hdd], pB.PotDiaHabil[hdd], alfa);
      1: PHoraria[iHoraDelPaso] :=
          ponder(pA.PotDiaMedioFeriado[hdd], pB.PotDiaMedioFeriado[hdd], alfa);
      2: PHoraria[iHoraDelPaso] :=
          ponder(pA.PotDiaFeriado[hdd], pB.PotDiaFeriado[hdd], alfa);
    end; // del case
    hdd := (hdd + 1) mod 24;
  end;
  inherited prepararPaso_as;
end;


procedure TDemanda01.Free;
begin
  self.lpd.Free;
  inherited Free;
end;

class function TDemanda01.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TDemanda01, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('lpd', TDemanda01(nil).lpd, TDataTypeDataRowOfCosa, 1, -1,TFichasLPD);

  Result:=list;

end;
//-------------------------
// Métodos de TFichaDemanda
//=========================

constructor TFichaDemanda01.Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
  PotDiaHabil, PotDiaMedioFeriado, PotDiaFeriado: TDAOfNreal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.PotDiaHabil := copy(PotDiaHabil, 0, MAXINT);
  self.PotDiaMedioFeriado := copy(PotDiaMedioFeriado, 0, MAXINT);
  self.PotDiaFeriado := copy(PotDiaFeriado, 0, MAXINT);
end;



function aplicarCrecimientos_CurvaHoraria(var PotHoraria: TDAOfNReal;
  fce, fcp: NReal): boolean;
var
  k: integer;
  gamma: NReal;
  E, Pp, Pk, PpPrima: NReal;
  res: boolean;
  Pm: NReal;
  PMin: NReal;
  ro: TDAOfNReal; // ro_i = Pi/Pp


begin
  E := 0;
  Pp := PotHoraria[0];
  PMin:= PotHoraria[0];
  for k := 0 to high(POtHOraria) do
  begin
    Pk := PotHoraria[k];
    E := E + Pk;
    if Pk > Pp then
      Pp := Pk
    else if Pk < PMin then
      PMin:= Pk;
  end;
  Pm := E / length(PotHoraria);

  res:= true;
  if (EsCero(Pp - Pm)) then
  begin
    // es una demanda plana, aplico solo factor de energía
    for k := 0 to high(PotHoraria) do
    begin
      PotHoraria[k] := PotHoraria[k] * fce;
    end;
  end
  else
  begin
    gamma := (fcp * Pp - fce * Pm) / (Pp - Pm);
    PpPrima := fcp * Pp;
    if ( gamma >= 0 ) and (( PpPrima - gamma * (PP - Pmin ))>=0) then
    begin
       for k := 0 to high( PotHoraria) do
       begin
         Pk := PpPrima - gamma * (PP - PotHoraria[k]);
         PotHoraria[k] := Pk;
       end;
    end
    else
      res:= false;
  end;
  Result := res;
end;

constructor TFichaDemanda01.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('PotDiaHabil', PotDiaHabil);
  f.rd('PotDiaMedioFeriado', PotDiaMedioFeriado);
  f.rd('PotDiaFeriado', PotDiaFeriado);
  f.EjecutarLectura;
end;


procedure TFichaDemanda01.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('PotDiaHabil', PotDiaHabil, uconstantesSimSEE.CF_PRECISION,
    uconstantesSimSEE.CF_DECIMALES);
  f.wr('PotDiaMedioFeriado', PotDiaMedioFeriado, uconstantesSimSEE.CF_PRECISION,
    uconstantesSimSEE.CF_DECIMALES);
  f.wr('PotDiaFeriado', PotDiaFeriado, uconstantesSimSEE.CF_PRECISION,
    uconstantesSimSEE.CF_DECIMALES);
end;

function TFichaDemanda01.infoAd_: string;
begin
  Result := 'PotDíaHábil Media= ' + FloatToStrF(vprom(PotDiaHabil), ffGeneral, 10, 1) +
    ' MW, ' + 'PotDíaMedioFeriado Media= ' +
    FloatToStrF(vprom(PotDiaMedioFeriado), ffGeneral, 10, 1) + ' MW, ' +
    'PotDíaFeriado Media= ' + FloatToStrF(vprom(PotDiaFeriado),
    ffGeneral, 10, 1) + ' MW';
end;

procedure TFichaDemanda01.Free;
begin
  setlength(PotDiaHabil, 0);
  setlength(PotDiaMedioFeriado, 0);
  setlength(PotDiaFeriado, 0);
  inherited Free;
end;

class function TFichaDemanda01.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TFichaDemanda01, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('PotDiaHabil', TFichaDemanda01(nil).PotDiaHabil, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PotDiaMedioFeriado', TFichaDemanda01(nil).PotDiaMedioFeriado, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PotDiaFeriado', TFichaDemanda01(nil).PotDiaFeriado, TDataTypeDoubleOfCosa, 1, -1);

  Result:=list;

end;



procedure AlInicio;
begin
  registrarClaseDeCosa(TDemanda01.ClassName, TDemanda01);
  registrarClaseDeCosa(TFichaDemanda01.ClassName, TFichaDemanda01);
end;

procedure AlFinal;
begin
end;

end.

